#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <stdint.h>

#include <dingoo/ucos2.h>
#include <dingoo/entry.h>

#include <sml/graphics.h>
#include <sml/sprite.h>
#include <sml/display.h>
#include <sml/control.h>
#include <sml/timer.h>
#include <sml/fixmath.h>


#define VERSION	1.1


gfx_font	*gameFont1    = NULL,
		*gameFont2    = NULL;
display		*gameDisplay  = NULL;
timer		*gameTimer    = NULL;
uint32_t	gameTickRate = (timer_resolution / 30);
bool		gameRunning  = true;
uintptr_t	i, n;
char		tempString[FILENAME_MAX];

#define		DICE_COUNT 9
#define		NUMBER_MAX 16
#define		NUMBER_MAX2 8

char *dices[] =	{"dF", "d2", "d4", "d6", "d8", "d10", "d12", "d20", "d100"};
uint32_t	number = 1, dice = 3;
int32_t		result = 0,
		details[NUMBER_MAX];
bool		bResult = false,
		bDetails = false;


void roll() {
	result = 0;
	for(i=0; i<number; i++) {
		details[i] = 0;
		switch(dice) {
			case 0: // dF
				details[i] = rand()%3-1;
				break;
			case 1: // d2
				details[i] = rand()%2+1;
				break;
			case 2: // d4
				details[i] = rand()%4+1;
				break;
			case 3: // d6
				details[i] = rand()%6+1;
				break;
			case 4: // d8
				details[i] = rand()%8+1;
				break;
			case 5: // d10
				details[i] = rand()%10+1;
				break;
			case 6: // d12
				details[i] = rand()%12+1;
				break;
			case 7: // d20
				details[i] = rand()%20+1;
				break;
			case 8: // d100
				details[i] = (rand()%10)*10 + (rand()%10+1);
				break;
			default: break;
		}
		result += details[i];
	}
	bResult = true;
}


int main(int argc, char** argv) {
	int ref = EXIT_SUCCESS;

	srand(OSTimeGet());

	control_init();
	gameDisplay = display_create(320, 240, 320, (DISPLAY_FORMAT_RGB565 | DISPLAY_BUFFER_STATIC), NULL, NULL);
	if(gameDisplay == NULL) {
		control_term();
		return ref;
	}
	gfx_init(gameDisplay);

	gfx_render_target_clear(gfx_color_rgb(0x00, 0x00, 0x00));
	display_flip(gameDisplay);
	control_lock(timer_resolution / 4);

	gameFont1 = gfx_font_load("font1.tga", COLOR_BLACK);
	gameFont2 = gfx_font_load("font2.tga", COLOR_BLACK);

	const uint32_t	y_0 = 4,
			y_1 = 68  - gfx_font_height(gameFont1)/2,
			y_2 = 120 - gfx_font_height(gameFont1)/2,
			y_3 = 165 - gfx_font_height(gameFont2)/2,
			y_4 = y_3 + 4 + gfx_font_height(gameFont2);

	gameTimer = timer_create();

	uint32_t tempTick    = 0;
	uint32_t tempPauseOS = 0;

	int sysref;
	while(gameRunning) {
		sysref = _sys_judge_event(NULL);
		if(sysref < 0) {
			ref = sysref;
			break;
		}

		tempTick += timer_delta(gameTimer);
		if(tempTick < gameTickRate) {
			tempPauseOS = (((gameTickRate - tempTick) * OS_TICKS_PER_SEC) / timer_resolution);
			if(tempPauseOS > 0) OSTimeDly(tempPauseOS);
			while(tempTick < gameTickRate)
				tempTick += timer_delta(gameTimer);
		}


		if(tempTick > (gameTickRate << 2))
			tempTick = 0; // HACK - Fixes timing glitch of unknown origin.
		while(tempTick >= gameTickRate) {
			tempTick -= gameTickRate;

			control_poll();
			if(control_check(CONTROL_BUTTON_X).pressed && control_check(CONTROL_BUTTON_Y).pressed) {
				// exit
				gameRunning = false;
				break;
			}
			if(control_check(CONTROL_BUTTON_B).pressed && control_check(CONTROL_BUTTON_B).changed) {
				// show/hide details
				if(bDetails)
					bDetails = false;
				else
					bDetails = true;
			}
			if(control_check(CONTROL_DPAD_RIGHT).pressed && control_check(CONTROL_DPAD_RIGHT).changed) {
				// next dice
				if(dice < DICE_COUNT-1) {
					bResult = false;
					dice++;
				}
			}
			if(control_check(CONTROL_DPAD_LEFT).pressed && control_check(CONTROL_DPAD_LEFT).changed) {
				// prev dice
				if(dice > 0) {
					bResult = false;
					dice--;
				}
			}
			if(control_check(CONTROL_DPAD_UP).pressed && control_check(CONTROL_DPAD_UP).changed) {
				// inc number
				if(number < NUMBER_MAX) {
					bResult = false;
					number++;
				}
			}
			if(control_check(CONTROL_DPAD_DOWN).pressed && control_check(CONTROL_DPAD_DOWN).changed) {
				// dec number
				if(number > 1) {
					bResult = false;
					number--;
				}
			}
			if(control_check(CONTROL_BUTTON_A).pressed && control_check(CONTROL_BUTTON_A).changed) {
				// roll
				roll();
			}

			gfx_render_target_clear(gfx_color_rgb(0x00, 0x00, 0x00));

			// Title
			sprintf(tempString, "Dicer v%.1f", VERSION);
			gfx_font_print_center(y_0, gameFont1, tempString);

			// roll type
			sprintf(tempString, "%u%s", number, dices[dice]);
			gfx_font_print_center(y_1, gameFont1, tempString);

			// roll result
			if(bResult) {
				sprintf(tempString, "%d", result);
				gfx_font_print_center(y_2, gameFont1, tempString);
				
				// details
				if(bDetails) {
					sprintf(tempString, "%d", details[0]);
					if(number <= NUMBER_MAX2) {
						for(i=1; i<number; i++)
							sprintf(tempString, "%s  %d", tempString, details[i]);
						gfx_font_print_center(y_3, gameFont2, tempString);
					}
					else {
						n = number / 2;
						for(i=1; i<n; i++)
							sprintf(tempString, "%s  %d", tempString, details[i]);
						gfx_font_print_center(y_3, gameFont2, tempString);
						sprintf(tempString, "%d", details[n]);
						for(i=n+1; i<number; i++)
							sprintf(tempString, "%s  %d", tempString, details[i]);
						gfx_font_print_center(y_4, gameFont2, tempString);
					}
				}
			}
			display_flip(gameDisplay);
		}
	}

	timer_delete(gameTimer);
	gfx_font_delete(gameFont1);
	gfx_font_delete(gameFont2);

	gfx_term();
	display_delete(gameDisplay);
	control_term();

	return 0;
}
